==> anonnet-0.0.1-20011210.tgz: anonnet/src/job.c <==
/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "common.h"
#include "conf.h"
#include "log.h"
#include "object.h"
#include "job.h"
#include "queue.h"


void scheduler_init() {
	object_init(&__schedule_mutex,NULL,NULL);
	SLIST_INIT(&__jobs);
	
	return;
}


void schedule(void(*func)(void *),void *arg,int sec,bool immediately,bool fork) {
	struct job_ctx *job;


	ASSERT((job = _malloc(sizeof(struct job_ctx))) != NULL,"unable to allocate new job");
	ASSERT(sec > 0,"bad argument in %s",__func__);

	job->__job	= func;
	job->__arg	= arg;
	job->__fork	= fork;
	job->__every	= sec;
	job->__last	= (immediately)? 0 : time(NULL);

	object_lock(&__schedule_mutex);

	SLIST_INSERT_HEAD(&__jobs,job,__sle);

	object_unlock(&__schedule_mutex);

	return;
}


void scheduler() {
	struct job_ctx *job;
	thread_t *thr;

	DEBUG("scheduler started");

	for (;;) {
		usleep(SCHEDULE_SLEEP);

		object_lock(&__schedule_mutex);

		SLIST_FOREACH(job,&__jobs,__sle) {
			if ((time(NULL) - job->__last) > job->__every) {
				if (job->__fork) {
					ASSERT((thr = thread_create(job->__job,job->__arg)) != NULL,
						"could not create job thread");
					thread_run(thr);
				} else {
					job->__job(job->__arg);
				}
				
				job->__last = time(NULL);
			}
		}	

		object_unlock(&__schedule_mutex);
	}

}

==> anonnet-0.0.1-20021020.tgz: anonnet/src/job.c <==
/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>

#include "common.h"
#include "conf.h"
#include "log.h"
#include "object.h"
#include "job.h"
#include "queue.h"


pthread_mutex_t jobs_mutex	= PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t	jobs_cond	= PTHREAD_COND_INITIALIZER;
SLIST_HEAD(,job_ctx) jobs	= SLIST_HEAD_INITIALIZER(struct jobs);

void schedule(void(*func)(void *),void *arg,int sec,bool immediately,bool fork) {
	struct job_ctx *job;


	ASSERT((job = _malloc(sizeof(struct job_ctx))) != NULL,"unable to allocate new job");

	job->__job	= func;
	job->__arg	= arg;
	job->__fork	= fork;
	job->__every	= MAX(sec,SCHEDULE_SLEEP);
	job->__last	= (immediately)? 0 : time(NULL);

	pthread_mutex_lock(&jobs_mutex);

	SLIST_INSERT_HEAD(&jobs,job,__sle);
	pthread_cond_broadcast(&jobs_cond);

	pthread_mutex_unlock(&jobs_mutex);

	return;
}


void scheduler() {
	struct job_ctx *job;
	thread_t *thr;
	int slp;

	DEBUG("scheduler started");

	for (;;) {

		pthread_mutex_lock(&jobs_mutex);
		
		if (SLIST_EMPTY(&jobs))
			ASSERT(pthread_cond_wait(&jobs_cond,&jobs_mutex) == 0,
				"pthread_cond_wait failed!");

		SLIST_FOREACH(job,&jobs,__sle) {
			if ((time(NULL) - job->__last) > job->__every) {
				if (job->__fork) {
					ASSERT((thr = thread_create(job->__job,job->__arg)) != NULL,
						"could not create job thread");
					thread_run(thr);
				} else {
					job->__job(job->__arg);
				}
				
				job->__last = time(NULL);
			}
		}	

		pthread_mutex_unlock(&jobs_mutex);

		usleep(SCHEDULE_SLEEP);
	}

}
